@@ -1,8 +1,10 @@
-from .exceptions import AllocationError, NotEnoughChunks
+from .exceptions import (
+    AllocationError,
+    NotEnoughChunks,
+)
 from .malloc import MallocPointer
 from ._cstd import c_calloc
 from typing import Iterator, Optional, Dict, TypeVar, Generic
-import ctypes
 
 __all__ = ("CallocPointer", "calloc")
 
@@ -25,18 +27,14 @@ class CallocPointer(MallocPointer, Generic[T]):
         self._chunk_size = chunk_size
         self._current_index = current_index
         self._chunks = chunks
-        self._freed = False
         self._chunk_cache = chunk_cache or {0: self}
         self._assigned = True
+        self._tracked = False
+        self._freed = False
 
         if chunk_cache:
             self._chunk_cache[self.current_index] = self
 
-        bytes_a = (ctypes.c_ubyte * 24) \
-            .from_address(id(0))
-
-        ctypes.memmove(address, bytes_a, len(bytes_a))
-
     def dereference(self) -> T:
         """Dereference the pointer."""
 
