@@ -1,18 +1,24 @@
 import ctypes
-from typing import (
-    Generic,
-    TypeVar,
-    Type,
-    Iterator,
-    Union,
-    Any
-)
-
+from typing import Generic, TypeVar, Type, Iterator, Union, Any
 from typing_extensions import ParamSpec
 from contextlib import suppress
 import faulthandler
 from io import UnsupportedOperation
 import sys
+from _pointers import add_ref, remove_ref, force_set_attr as _force_set_attr
+from .exceptions import InvalidSizeError
+
+
+__all__ = ("Pointer", "to_ptr", "dereference_address")
+
+T = TypeVar("T")
+A = TypeVar("A")
+P = ParamSpec("P")
+
+with suppress(
+    UnsupportedOperation
+):  # in case its running in idle or something like that
+    faulthandler.enable()
 
 
 def dereference_address(address: int) -> Any:
@@ -20,25 +26,36 @@ def dereference_address(address: int) -> Any:
     return ctypes.cast(address, ctypes.py_object).value
 
 
-with suppress(
-    UnsupportedOperation
-):  # in case its running in idle or something like that
-    faulthandler.enable()
+def force_set_attr(typ: Type[Any], key: str, value: Any) -> None:
+    """Force setting an attribute on the target type."""
 
-__all__ = ("Pointer", "to_ptr")
+    if not isinstance(typ, type):
+        raise ValueError(
+            f"{typ} does not derive from type (did you pass an instance and not a class)?",  # noqa
+        )
 
-T = TypeVar("T")
-A = TypeVar("A")
-P = ParamSpec("P")
+    _force_set_attr(typ, key, value)
+
+
+def _make_ptr(data: Union["Pointer[T]", T]) -> "Pointer[T]":
+    return data if isinstance(data, Pointer) else to_ptr(data)
 
 
 class Pointer(Generic[T]):
     """Base class representing a pointer."""
 
-    def __init__(self, address: int, typ: Type[T]) -> None:
+    def __init__(
+        self,
+        address: int,
+        typ: Type[T],
+        increment_ref: bool = False,
+    ) -> None:
         self._address = address
         self._type = typ
 
+        if increment_ref:
+            add_ref(~self)
+
     @property
     def address(self) -> int:
         """Address of the pointer."""
@@ -84,7 +101,7 @@ class Pointer(Generic[T]):
         self.assign(value if isinstance(value, Pointer) else to_ptr(value))
         return self
 
-    def move(self, data: "Pointer[T]") -> None:
+    def move(self, data: "Pointer[T]", unsafe: bool = False) -> None:
         """Move data from another pointer to this pointer. Very dangerous, use with caution."""  # noqa
         if data.type is not self.type:
             raise ValueError("pointer must be the same type")
@@ -92,25 +109,45 @@ class Pointer(Generic[T]):
         deref_a: T = ~data
         deref_b: T = ~self
 
-        bytes_a = (ctypes.c_ubyte * sys.getsizeof(deref_a)) \
-            .from_address(data.address)
-        bytes_b = (ctypes.c_ubyte * sys.getsizeof(deref_b)) \
-            .from_address(self.address)
+        size_a: int = sys.getsizeof(deref_a)
+        size_b: int = sys.getsizeof(deref_b)
+
+        if (size_b < size_a) and (not unsafe):
+            raise InvalidSizeError(
+                f"target size may not exceed current size ({size_a} < {size_b})",  # noqa
+            )
+
+        bytes_a = (ctypes.c_ubyte * size_a).from_address(data.address)
+        bytes_b = (ctypes.c_ubyte * size_b).from_address(self.address)
 
         ctypes.memmove(bytes_b, bytes_a, len(bytes_a))
 
     def __lshift__(self, data: Union["Pointer[T]", T]):
-        """Move data from another pointer to this pointer. Very dangerous, use with caution."""  # noqa
-        self.move(data if isinstance(data, Pointer) else to_ptr(data))
+        self.move(_make_ptr(data))
         return self
 
-    def __add__(self, amount: int):
-        return Pointer(self.address + (amount * 24), self.type)
+    def __xor__(self, data: Union["Pointer[T]", T]):
+        self.move(_make_ptr(data), unsafe=True)
+        return self
 
-    def __sub__(self, amount: int):
-        return Pointer(self.address - (amount * 24), self.type)
+    def __del__(self) -> None:
+        remove_ref(~self)
+
+    def __eq__(self, data: object) -> bool:
+        if not isinstance(data, Pointer):
+            return False
+
+        return data.address == self.address
+
+    def set_attr(self, key: str, value: Any) -> None:
+        v: Any = ~self  # mypy gets angry if this isnt any
+        if not isinstance(~self, type):
+            v = type(v)
+
+        force_set_attr(v, key, value)
 
 
 def to_ptr(val: T) -> Pointer[T]:
     """Convert a value to a pointer."""
+    add_ref(val)
     return Pointer(id(val), type(val))
